#include <windows.h>
#include <cstdio>

void append_section (char *buffer, const char *label, int value)
{
	if (value) sprintf (buffer + strlen (buffer), "%d %s%s, ", value, label, (value > 1 ? "s" : ""));
}

DWORD get_uptime ()
{
	//For reading from registry
	PERF_DATA_BLOCK data = { 0 };
	DWORD dwSize = sizeof (data);
	DWORD dwType = 0;

	//Attempt to read from registry, adjusting buffer size
	RegQueryValueEx (HKEY_PERFORMANCE_DATA, "Global", NULL, &dwType, (LPBYTE)&data, &dwSize);
	
	//Check whether the data is in the required format
	if (wcsncmp (data.Signature, L"PERF", 4) == 0)
	{
		//Get value in 100nsec units and store as number of seconds
		return (DWORD)(data.PerfTime100nSec.QuadPart / (LONGLONG)(10 * 1000 * 1000));
	} else {
		//Performance counter isn't available -- use tickcount (which wraps every 49.7 days)
		return GetTickCount () / 1000;
	}
}

void prepare_text (char *buffer)
{
	//Get the uptime in seconds
	DWORD dwTime = get_uptime ();
	
	//Split into parts
	int days, hours, minutes, seconds;
	seconds = dwTime % 60;
	dwTime /= 60;
	minutes = dwTime % 60;
	dwTime /= 60;
	hours = dwTime % 24;
	dwTime /= 24;
	days = dwTime;
	
	//Compile message
	strcpy (buffer, "System has been up for ");
	append_section (buffer, "day", days);
	append_section (buffer, "hour", hours);
	append_section (buffer, "minute", minutes);
	append_section (buffer, "second", seconds);
	buffer [strlen (buffer) - 2] = 0;
}

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	//Get the uptime as a string
	char buffer [1024];
	prepare_text (buffer);

	//Display that in a messagebox
	MessageBox (NULL, buffer, "System Uptime", MB_ICONINFORMATION);
	
	return 0;
}
